﻿namespace AP
{
    using System;
    using System.Runtime.InteropServices;

    [StructLayout(LayoutKind.Sequential)]
    public struct Complex
    {
        public double x;
        public double y;
        public Complex(double _x)
        {
            this.x = _x;
            this.y = 0.0;
        }

        public Complex(double _x, double _y)
        {
            this.x = _x;
            this.y = _y;
        }

        public static implicit operator Complex(double _x)
        {
            return new Complex(_x);
        }

        public static bool operator ==(Complex lhs, Complex rhs)
        {
            return ((lhs.x == rhs.x) & (lhs.y == rhs.y));
        }

        public static bool operator !=(Complex lhs, Complex rhs)
        {
            return (!(lhs.x == rhs.x) | !(lhs.y == rhs.y));
        }

        public static Complex operator +(Complex lhs)
        {
            return lhs;
        }

        public static Complex operator -(Complex lhs)
        {
            return new Complex(-lhs.x, -lhs.y);
        }

        public static Complex operator +(Complex lhs, Complex rhs)
        {
            return new Complex(lhs.x + rhs.x, lhs.y + rhs.y);
        }

        public static Complex operator -(Complex lhs, Complex rhs)
        {
            return new Complex(lhs.x - rhs.x, lhs.y - rhs.y);
        }

        public static Complex operator *(Complex lhs, Complex rhs)
        {
            return new Complex((lhs.x * rhs.x) - (lhs.y * rhs.y), (lhs.x * rhs.y) + (lhs.y * rhs.x));
        }

        public static Complex operator /(Complex lhs, Complex rhs)
        {
            Complex complex;
            double num;
            double num2;
            if (System.Math.Abs(rhs.y) < System.Math.Abs(rhs.x))
            {
                num = rhs.y / rhs.x;
                num2 = rhs.x + (rhs.y * num);
                complex.x = (lhs.x + (lhs.y * num)) / num2;
                complex.y = (lhs.y - (lhs.x * num)) / num2;
                return complex;
            }
            num = rhs.x / rhs.y;
            num2 = rhs.y + (rhs.x * num);
            complex.x = (lhs.y + (lhs.x * num)) / num2;
            complex.y = (-lhs.x + (lhs.y * num)) / num2;
            return complex;
        }

        public override int GetHashCode()
        {
            return (this.x.GetHashCode() ^ this.y.GetHashCode());
        }

        public override bool Equals(object obj)
        {
            if (obj is byte)
            {
                return this.Equals(new Complex((double) ((byte) obj)));
            }
            if (obj is sbyte)
            {
                return this.Equals(new Complex((double) ((sbyte) obj)));
            }
            if (obj is short)
            {
                return this.Equals(new Complex((double) ((short) obj)));
            }
            if (obj is ushort)
            {
                return this.Equals(new Complex((double) ((ushort) obj)));
            }
            if (obj is int)
            {
                return this.Equals(new Complex((double) ((int) obj)));
            }
            if (obj is uint)
            {
                return this.Equals(new Complex((double) ((uint) obj)));
            }
            if (obj is long)
            {
                return this.Equals(new Complex((double) ((long) obj)));
            }
            if (obj is ulong)
            {
                return this.Equals(new Complex((double) ((ulong) obj)));
            }
            if (obj is float)
            {
                return this.Equals(new Complex((double) ((float) obj)));
            }
            if (obj is double)
            {
                return this.Equals(new Complex((double) obj));
            }
            if (obj is decimal)
            {
                return this.Equals(new Complex((double) ((decimal) obj)));
            }
            return base.Equals(obj);
        }
    }
}

