﻿namespace inv_nd
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.CompilerServices;
    using TSLab.Script.Handlers;

    [HandlerCategory("777")]
    [HandlerName("InvNormalDistr")]
    public class InvNormalDistr : IThreeSourcesHandler, IDoubleReturns, IStreamHandler, IDoubleInputs, IContextUses
    {
        public const double DBL_EPSILON = 1E-07;
        public int log_p = 0;
        public int lower_tail = 1;

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3)
        {
            double p = 0.0;
            double mu = 0.0;
            double sigma = 0.0;
            double item = 0.0;
            IList<double> list = new List<double>(source1.Count);
            for (int i = 0; i < source1.Count; i++)
            {
                p = source1[i];
                mu = source2[i];
                sigma = source3[i];
                item = this.qnorm(p, mu, sigma);
                list.Add(item);
            }
            return list;
        }

        private double expm1(double x)
        {
            double num;
            double num2 = Math.Abs(x);
            if (num2 < 1E-07)
            {
                return x;
            }
            if (num2 > 0.697)
            {
                return (Math.Exp(x) - 1.0);
            }
            if (num2 > 1E-08)
            {
                num = Math.Exp(x) - 1.0;
            }
            else
            {
                num = ((x / 2.0) + 1.0) * x;
            }
            return (num - ((1.0 + num) * (Math.Log(1.0 + num) - x)));
        }

        private double ML_NEGINF()
        {
            return double.NegativeInfinity;
        }

        private double ML_POSINF()
        {
            return double.NegativeInfinity;
        }

        private double qnorm(double p, double mu, double sigma)
        {
            double num3;
            double num4;
            this.lower_tail = 1;
            this.log_p = 0;
            if (p == this.R_DT_0())
            {
                return this.ML_NEGINF();
            }
            if (p == this.R_DT_1())
            {
                return this.ML_POSINF();
            }
            if (this.R_Q_P01_check(p))
            {
                return 0.0;
            }
            if (sigma < 0.0)
            {
                return 0.0;
            }
            if (sigma == 0.0)
            {
                return mu;
            }
            double num = this.R_DT_qIv(p);
            double num2 = num - 0.5;
            if (Math.Abs(num2) <= 0.425)
            {
                num3 = 0.180625 - (num2 * num2);
                num4 = (num2 * ((((((((((((((num3 * 2509.0809287301227) + 33430.575583588128) * num3) + 67265.7709270087) * num3) + 45921.95393154987) * num3) + 13731.693765509461) * num3) + 1971.5909503065513) * num3) + 133.14166789178438) * num3) + 3.3871328727963665)) / ((((((((((((((num3 * 5226.4952788528544) + 28729.085735721943) * num3) + 39307.895800092709) * num3) + 21213.794301586597) * num3) + 5394.1960214247511) * num3) + 687.18700749205789) * num3) + 42.313330701600911) * num3) + 1.0);
            }
            else
            {
                if (num2 > 0.0)
                {
                    num3 = this.R_DT_CIv(p);
                }
                else
                {
                    num3 = num;
                }
                num3 = Math.Sqrt(-(((this.log_p != 0) && (((this.lower_tail != 0) && (num2 <= 0.0)) || ((this.lower_tail == 0) && (num2 > 0.0)))) ? p : Math.Log(num3)));
                if (num3 <= 5.0)
                {
                    num3 += -1.6;
                    num4 = ((((((((((((((num3 * 0.00077454501427834139) + 0.022723844989269184) * num3) + 0.24178072517745061) * num3) + 1.2704582524523684) * num3) + 3.6478483247632045) * num3) + 5.769497221460691) * num3) + 4.6303378461565456) * num3) + 1.4234371107496835) / ((((((((((((((num3 * 1.0507500716444169E-09) + 0.00054759380849953455) * num3) + 0.015198666563616457) * num3) + 0.14810397642748008) * num3) + 0.6897673349851) * num3) + 1.6763848301838038) * num3) + 2.053191626637759) * num3) + 1.0);
                }
                else
                {
                    num3 += -5.0;
                    num4 = ((((((((((((((num3 * 2.0103343992922881E-07) + 2.7115555687434876E-05) * num3) + 0.0012426609473880784) * num3) + 0.026532189526576124) * num3) + 0.29656057182850487) * num3) + 1.7848265399172913) * num3) + 5.4637849111641144) * num3) + 6.6579046435011033) / ((((((((((((((num3 * 2.0442631033899397E-15) + 1.4215117583164459E-07) * num3) + 1.8463183175100548E-05) * num3) + 0.00078686913114561329) * num3) + 0.014875361290850615) * num3) + 0.13692988092273581) * num3) + 0.599832206555888) * num3) + 1.0);
                }
                if (num2 < 0.0)
                {
                    num4 = -num4;
                }
            }
            return (mu + (sigma * num4));
        }

        private double R_D__0()
        {
            return ((this.log_p != 0) ? this.ML_NEGINF() : 0.0);
        }

        private double R_D__1()
        {
            return ((this.log_p != 0) ? 0.0 : 1.0);
        }

        private double R_D_Cval(double p)
        {
            return ((this.lower_tail != 0) ? (1.0 - p) : p);
        }

        private double R_D_Lval(double p)
        {
            return ((this.lower_tail != 0) ? p : (1.0 - p));
        }

        private double R_DT_0()
        {
            return ((this.lower_tail != 0) ? this.R_D__0() : this.R_D__1());
        }

        private double R_DT_1()
        {
            return ((this.lower_tail != 0) ? this.R_D__1() : this.R_D__0());
        }

        private double R_DT_CIv(double p)
        {
            return ((this.log_p != 0) ? ((this.lower_tail != 0) ? -this.expm1(p) : Math.Exp(p)) : this.R_D_Cval(p));
        }

        private double R_DT_qIv(double p)
        {
            return ((this.log_p != 0) ? ((this.lower_tail != 0) ? Math.Exp(p) : -this.expm1(p)) : this.R_D_Lval(p));
        }

        private bool R_Q_P01_check(double p)
        {
            return (((this.log_p != 0) && (p > 0.0)) || ((this.log_p == 0) && ((p < 0.0) || (p > 1.0))));
        }

        public IContext Context { get; set; }
    }
}

